searchNodes=[{"doc":"Poison Poison is a new JSON library for Elixir focusing on wicked-fast speed without sacrificing simplicity , completeness , or correctness . Poison takes several approaches to be the fastest JSON library for Elixir. Poison uses extensive sub binary matching , a hand-rolled parser using several techniques that are known to benefit BeamAsm for JIT compilation, IO list encoding and single-pass decoding. Poison benchmarks sometimes puts Poison's performance close to jiffy and usually faster than other Erlang/Elixir libraries. Poison fully conforms to RFC 7159 , ECMA 404 , and fully passes the JSONTestSuite . Installation First, add Poison to your mix.exs dependencies: def deps do [ { :poison , &quot;~&gt; 5.0&quot; } ] end Then, update your dependencies: $ mix deps.get Usage Poison . encode! ( %{ &quot;age&quot; =&gt; 27 , &quot;name&quot; =&gt; &quot;Devin Torres&quot; } ) #=&gt; &quot;{\\&quot;name\\&quot;:\\&quot;Devin Torres\\&quot;,\\&quot;age\\&quot;:27}&quot; Poison . decode! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) ) #=&gt; %{&quot;age&quot; =&gt; 27, &quot;name&quot; =&gt; &quot;Devin Torres&quot;} defmodule Person do @derive [ Poison.Encoder ] defstruct [ :name , :age ] end Poison . encode! ( % Person { name : &quot;Devin Torres&quot; , age : 27 } ) #=&gt; &quot;{\\&quot;name\\&quot;:\\&quot;Devin Torres\\&quot;,\\&quot;age\\&quot;:27}&quot; Poison . decode! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) , as : % Person { } ) #=&gt; %Person{name: &quot;Devin Torres&quot;, age: 27} Poison . decode! ( ~s({&quot;people&quot;: [{&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}]}) , as : %{ &quot;people&quot; =&gt; [ % Person { } ] } ) #=&gt; %{&quot;people&quot; =&gt; [%Person{age: 27, name: &quot;Devin Torres&quot;}]} Every component of Poison (encoder, decoder, and parser) are all usable on their own without buying into other functionality. For example, if you were interested purely in the speed of parsing JSON without a decoding step, you could simply call Poison.Parser.parse . Parser iex&gt; Poison.Parser . parse! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) , %{ } ) %{ &quot;name&quot; =&gt; &quot;Devin Torres&quot; , &quot;age&quot; =&gt; 27 } iex&gt; Poison.Parser . parse! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) , %{ keys : :atoms! } ) %{ name : &quot;Devin Torres&quot; , age : 27 } Note that keys: :atoms! reuses existing atoms, i.e. if :name was not allocated before the call, you will encounter an argument error message. You can use the keys: :atoms variant to make sure all atoms are created as needed. However, unless you absolutely know what you're doing, do not do it. Atoms are not garbage-collected, see Erlang Efficiency Guide for more info: Atoms are not garbage-collected. Once an atom is created, it will never be removed. The emulator will terminate if the limit for the number of atoms (1048576 by default) is reached. Encoder iex&gt; Poison.Encoder . encode ( [ 1 , 2 , 3 ] , %{ } ) |&gt; IO . iodata_to_binary &quot;[1,2,3]&quot; Anything implementing the Encoder protocol is expected to return an IO list to be embedded within any other Encoder's implementation and passable to any IO subsystem without conversion. defimpl Poison.Encoder , for : Person do def encode ( %{ name : name , age : age } , options ) do Poison.Encoder.BitString . encode ( &quot; \#{ name } ( \#{ age } )&quot; , options ) end end For maximum performance, make sure you @derive [Poison.Encoder] for any struct you plan on encoding. Encoding only some attributes When deriving structs for encoding, it is possible to select or exclude specific attributes. This is achieved by deriving Poison.Encoder with the :only or :except options set: defmodule PersonOnlyName do @derive { Poison.Encoder , only : [ :name ] } defstruct [ :name , :age ] end defmodule PersonWithoutName do @derive { Poison.Encoder , except : [ :name ] } defstruct [ :name , :age ] end In case both :only and :except keys are defined, the :except option is ignored. Key Validation According to RFC 7159 keys in a JSON object should be unique. This is enforced and resolved in different ways in other libraries. In the Ruby JSON library for example, the output generated from encoding a hash with a duplicate key (say one is a string, the other an atom) will include both keys. When parsing JSON of this type, Chromium will override all previous values with the final one. Poison will generate JSON with duplicate keys if you attempt to encode a map with atom and string keys whose encoded names would clash. If you'd like to ensure that your generated JSON doesn't have this issue, you can pass the strict_keys: true option when encoding. This will force the encoding to fail. Note: Validating keys can cause a small performance hit. iex&gt; Poison . encode! ( %{ :foo =&gt; &quot;foo1&quot; , &quot;foo&quot; =&gt; &quot;foo2&quot; } , strict_keys : true ) ** (Poison.EncodeError) duplicate key found: :foo Benchmarking $ MIX_ENV=bench mix run bench/run.exs Current Benchmarks As of 2020-06-25: Amazon EC2 c5.2xlarge instance running Ubuntu Server 20.04: https://gist.github.com/devinus/c82c2f6eaa22456e7ff0f5705466b1de License Poison is released under the public-domain-equivalent 0BSD license.","ref":"Poison.html","title":"Poison","type":"module"},{"doc":"Decode JSON to a value. iex&gt; Poison . decode ( &quot;[1,2,3]&quot; ) { :ok , [ 1 , 2 , 3 ] }","ref":"Poison.html#decode/2","title":"Poison.decode/2","type":"function"},{"doc":"Decode JSON to a value, raises an exception on error. iex&gt; Poison . decode! ( &quot;[1,2,3]&quot; ) [ 1 , 2 , 3 ]","ref":"Poison.html#decode!/1","title":"Poison.decode!/1","type":"function"},{"doc":"","ref":"Poison.html#decode!/2","title":"Poison.decode!/2","type":"function"},{"doc":"Encode a value to JSON. iex&gt; Poison . encode ( [ 1 , 2 , 3 ] ) { :ok , &quot;[1,2,3]&quot; }","ref":"Poison.html#encode/2","title":"Poison.encode/2","type":"function"},{"doc":"Encode a value to JSON, raises an exception on error. iex&gt; Poison . encode! ( [ 1 , 2 , 3 ] ) &quot;[1,2,3]&quot;","ref":"Poison.html#encode!/2","title":"Poison.encode!/2","type":"function"},{"doc":"","ref":"Poison.Decoder.html","title":"Poison.Decoder","type":"protocol"},{"doc":"","ref":"Poison.Decoder.html#decode/2","title":"Poison.Decoder.decode/2","type":"function"},{"doc":"","ref":"Poison.Decoder.html#t:options/0","title":"Poison.Decoder.options/0","type":"type"},{"doc":"","ref":"Poison.Decoder.html#t:t/0","title":"Poison.Decoder.t/0","type":"type"},{"doc":"","ref":"Poison.Encoder.html","title":"Poison.Encoder","type":"protocol"},{"doc":"","ref":"Poison.Encoder.html#encode/2","title":"Poison.Encoder.encode/2","type":"function"},{"doc":"","ref":"Poison.Encoder.html#t:options/0","title":"Poison.Encoder.options/0","type":"type"},{"doc":"","ref":"Poison.Encoder.html#t:t/0","title":"Poison.Encoder.t/0","type":"type"},{"doc":"An RFC 7159 and ECMA 404 conforming JSON parser. See: https://tools.ietf.org/html/rfc7159 See: http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-404.pdf","ref":"Poison.Parser.html","title":"Poison.Parser","type":"module"},{"doc":"","ref":"Poison.Parser.html#parse!/2","title":"Poison.Parser.parse!/2","type":"function"},{"doc":"","ref":"Poison.Parser.html#t:t/0","title":"Poison.Parser.t/0","type":"type"},{"doc":"","ref":"Poison.DecodeError.html","title":"Poison.DecodeError","type":"exception"},{"doc":"","ref":"Poison.DecodeError.html#t:t/0","title":"Poison.DecodeError.t/0","type":"type"},{"doc":"","ref":"Poison.EncodeError.html","title":"Poison.EncodeError","type":"exception"},{"doc":"","ref":"Poison.EncodeError.html#t:t/0","title":"Poison.EncodeError.t/0","type":"type"},{"doc":"","ref":"Poison.MissingDependencyError.html","title":"Poison.MissingDependencyError","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"Poison.MissingDependencyError.html#message/1","title":"Poison.MissingDependencyError.message/1","type":"function"},{"doc":"","ref":"Poison.MissingDependencyError.html#t:t/0","title":"Poison.MissingDependencyError.t/0","type":"type"},{"doc":"","ref":"Poison.ParseError.html","title":"Poison.ParseError","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"Poison.ParseError.html#message/1","title":"Poison.ParseError.message/1","type":"function"},{"doc":"","ref":"Poison.ParseError.html#t:t/0","title":"Poison.ParseError.t/0","type":"type"},{"doc":"Poison Poison is a new JSON library for Elixir focusing on wicked-fast speed without sacrificing simplicity , completeness , or correctness . Poison takes several approaches to be the fastest JSON library for Elixir. Poison uses extensive sub binary matching , a hand-rolled parser using several techniques that are known to benefit BeamAsm for JIT compilation, IO list encoding and single-pass decoding. Poison benchmarks sometimes puts Poison's performance close to jiffy and usually faster than other Erlang/Elixir libraries. Poison fully conforms to RFC 7159 , ECMA 404 , and fully passes the JSONTestSuite .","ref":"readme.html","title":"Poison","type":"extras"},{"doc":"First, add Poison to your mix.exs dependencies: def deps do [ { :poison , &quot;~&gt; 5.0&quot; } ] end Then, update your dependencies: $ mix deps.get","ref":"readme.html#installation","title":"Poison - Installation","type":"extras"},{"doc":"Poison . encode! ( %{ &quot;age&quot; =&gt; 27 , &quot;name&quot; =&gt; &quot;Devin Torres&quot; } ) #=&gt; &quot;{\\&quot;name\\&quot;:\\&quot;Devin Torres\\&quot;,\\&quot;age\\&quot;:27}&quot; Poison . decode! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) ) #=&gt; %{&quot;age&quot; =&gt; 27, &quot;name&quot; =&gt; &quot;Devin Torres&quot;} defmodule Person do @derive [ Poison.Encoder ] defstruct [ :name , :age ] end Poison . encode! ( % Person { name : &quot;Devin Torres&quot; , age : 27 } ) #=&gt; &quot;{\\&quot;name\\&quot;:\\&quot;Devin Torres\\&quot;,\\&quot;age\\&quot;:27}&quot; Poison . decode! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) , as : % Person { } ) #=&gt; %Person{name: &quot;Devin Torres&quot;, age: 27} Poison . decode! ( ~s({&quot;people&quot;: [{&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}]}) , as : %{ &quot;people&quot; =&gt; [ % Person { } ] } ) #=&gt; %{&quot;people&quot; =&gt; [%Person{age: 27, name: &quot;Devin Torres&quot;}]} Every component of Poison (encoder, decoder, and parser) are all usable on their own without buying into other functionality. For example, if you were interested purely in the speed of parsing JSON without a decoding step, you could simply call Poison.Parser.parse .","ref":"readme.html#usage","title":"Poison - Usage","type":"extras"},{"doc":"iex&gt; Poison.Parser . parse! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) , %{ } ) %{ &quot;name&quot; =&gt; &quot;Devin Torres&quot; , &quot;age&quot; =&gt; 27 } iex&gt; Poison.Parser . parse! ( ~s({&quot;name&quot;: &quot;Devin Torres&quot;, &quot;age&quot;: 27}) , %{ keys : :atoms! } ) %{ name : &quot;Devin Torres&quot; , age : 27 } Note that keys: :atoms! reuses existing atoms, i.e. if :name was not allocated before the call, you will encounter an argument error message. You can use the keys: :atoms variant to make sure all atoms are created as needed. However, unless you absolutely know what you're doing, do not do it. Atoms are not garbage-collected, see Erlang Efficiency Guide for more info: Atoms are not garbage-collected. Once an atom is created, it will never be removed. The emulator will terminate if the limit for the number of atoms (1048576 by default) is reached.","ref":"readme.html#parser","title":"Poison - Parser","type":"extras"},{"doc":"iex&gt; Poison.Encoder . encode ( [ 1 , 2 , 3 ] , %{ } ) |&gt; IO . iodata_to_binary &quot;[1,2,3]&quot; Anything implementing the Encoder protocol is expected to return an IO list to be embedded within any other Encoder's implementation and passable to any IO subsystem without conversion. defimpl Poison.Encoder , for : Person do def encode ( %{ name : name , age : age } , options ) do Poison.Encoder.BitString . encode ( &quot; \#{ name } ( \#{ age } )&quot; , options ) end end For maximum performance, make sure you @derive [Poison.Encoder] for any struct you plan on encoding. Encoding only some attributes When deriving structs for encoding, it is possible to select or exclude specific attributes. This is achieved by deriving Poison.Encoder with the :only or :except options set: defmodule PersonOnlyName do @derive { Poison.Encoder , only : [ :name ] } defstruct [ :name , :age ] end defmodule PersonWithoutName do @derive { Poison.Encoder , except : [ :name ] } defstruct [ :name , :age ] end In case both :only and :except keys are defined, the :except option is ignored. Key Validation According to RFC 7159 keys in a JSON object should be unique. This is enforced and resolved in different ways in other libraries. In the Ruby JSON library for example, the output generated from encoding a hash with a duplicate key (say one is a string, the other an atom) will include both keys. When parsing JSON of this type, Chromium will override all previous values with the final one. Poison will generate JSON with duplicate keys if you attempt to encode a map with atom and string keys whose encoded names would clash. If you'd like to ensure that your generated JSON doesn't have this issue, you can pass the strict_keys: true option when encoding. This will force the encoding to fail. Note: Validating keys can cause a small performance hit. iex&gt; Poison . encode! ( %{ :foo =&gt; &quot;foo1&quot; , &quot;foo&quot; =&gt; &quot;foo2&quot; } , strict_keys : true ) ** (Poison.EncodeError) duplicate key found: :foo","ref":"readme.html#encoder","title":"Poison - Encoder","type":"extras"},{"doc":"$ MIX_ENV=bench mix run bench/run.exs Current Benchmarks As of 2020-06-25: Amazon EC2 c5.2xlarge instance running Ubuntu Server 20.04: https://gist.github.com/devinus/c82c2f6eaa22456e7ff0f5705466b1de","ref":"readme.html#benchmarking","title":"Poison - Benchmarking","type":"extras"},{"doc":"Poison is released under the public-domain-equivalent 0BSD license.","ref":"readme.html#license","title":"Poison - License","type":"extras"}]